@@ -110,6 +110,22 @@ namespace gcn
          */
         virtual BITMAP *getTarget();
 
+        /**
+         * Gets the color in Allegro format.
+         *
+         * @return the color in Allegro format.
+         */
+        int getAllegroColor() const;
+
+        /**
+         * Draws an Allegro bitmap.
+         *
+         * @param bitmap the bitmap to draw.
+         * @param dstX the x-coordinate where to draw the bitmap.
+         * @param dstY the y-coordinate where to draw the bitmap.
+         */
+        virtual void drawBitmap(BITMAP* bitmap, int dstX, int dstY);
+
 
         // Inherited from Graphics
 
@@ -121,8 +137,12 @@ namespace gcn
 
         virtual void popClipArea();
 
-        virtual void drawImage(const Image* image, int srcX, int srcY,
-                               int dstX, int dstY, int width,
+        virtual void drawImage(const Image* image,
+                               int srcX,
+                               int srcY,
+                               int dstX,
+                               int dstY,
+                               int width,
                                int height);
 
         virtual void drawPoint(int x, int y);
@@ -135,12 +155,12 @@ namespace gcn
 
         virtual void setColor(const Color& color);
 
-        virtual const Color& getColor();
+        virtual const Color& getColor() const;
 
     protected:
         BITMAP *mTarget;
         bool mClipNull;
-        int mAlColor;
+        int mAllegroColor;
         Color mColor;
     };
 }
