@@ -106,7 +106,8 @@ namespace gcn
          *
          * NOTE: The clip areas will be taken into account.
          */
-        virtual void drawSDLSurface(SDL_Surface* surface, SDL_Rect source,
+        virtual void drawSDLSurface(SDL_Surface* surface,
+                                    SDL_Rect source,
                                     SDL_Rect destination);
 
 
@@ -120,8 +121,12 @@ namespace gcn
 
         virtual void popClipArea();
 
-        virtual void drawImage(const Image* image, int srcX, int srcY,
-                               int dstX, int dstY, int width,
+        virtual void drawImage(const Image* image,
+                               int srcX,
+                               int srcY,
+                               int dstX,
+                               int dstY,
+                               int width,
                                int height);
 
         virtual void drawPoint(int x, int y);
@@ -134,7 +139,7 @@ namespace gcn
 
         virtual void setColor(const Color& color);
 
-        virtual const Color& getColor();
+        virtual const Color& getColor() const;
 
     protected:
         /**
