@@ -1,12 +1,14 @@
-/*      _______   __   __   __   ______   __   __   _______   __   __                 
- *     / _____/\ / /\ / /\ / /\ / ____/\ / /\ / /\ / ___  /\ /  |\/ /\                
- *    / /\____\// / // / // / // /\___\// /_// / // /\_/ / // , |/ / /                 
- *   / / /__   / / // / // / // / /    / ___  / // ___  / // /| ' / /                  
- *  / /_// /\ / /_// / // / // /_/_   / / // / // /\_/ / // / |  / /                   
- * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /                    
- * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/                      
+/*      _______   __   __   __   ______   __   __   _______   __   __
+ *     / _____/\ / /\ / /\ / /\ / ____/\ / /\ / /\ / ___  /\ /  |\/ /\
+ *    / /\____\// / // / // / // /\___\// /_// / // /\_/ / // , |/ / /
+ *   / / /__   / / // / // / // / /    / ___  / // ___  / // /| ' / /
+ *  / /_// /\ / /_// / // / // /_/_   / / // / // /\_/ / // / |  / /
+ * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /
+ * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/
  *
- * Copyright (c) 2004, 2005 darkbits                        Js_./
+ * Copyright (c) 2004, 2005, 2006, 2007 Olof Naessn and Per Larsson
+ *
+ *                                                         Js_./
  * Per Larsson a.k.a finalman                          _RqZ{a<^_aa
  * Olof Naessn a.k.a jansem/yakslem                _asww7!uY`>  )\a//
  *                                                 _Qhm`] _f "'c  1!5m
@@ -58,29 +60,23 @@
 #include <list>
 #include <string>
 
-#include "guichan/actionlistener.hpp"
 #include "guichan/color.hpp"
-#include "guichan/defaultfont.hpp"
-#include "guichan/font.hpp"
-#include "guichan/graphics.hpp"
-#include "guichan/keyinput.hpp"
-#include "guichan/keylistener.hpp"
-#include "guichan/mouseinput.hpp"
-#include "guichan/mouselistener.hpp"
 #include "guichan/rectangle.hpp"
-#include "guichan/platform.hpp"
 
 namespace gcn
 {
-    /*
-     * Exists for the widget class to compile.
-     */
-    class FocusHandler;
-
-    /*
-     * Exists for the widget class to compile.
-     */
+    class ActionListener;
     class BasicContainer;
+    class DeathListener;
+    class DefaultFont;
+    class FocusHandler;
+    class FocusListener;
+    class Font;
+    class Graphics;
+    class KeyInput;
+    class KeyListener;
+    class MouseInput;
+    class MouseListener;
 
     /**
      * Widget base class. Contains basic widget functions every widget should
@@ -88,262 +84,248 @@ namespace gcn
      * functions.
      *
      * NOTE: Functions begining with underscore "_" should not
-     *       be overloaded unless you know what you are doing.
+     *       be overloaded unless you know what you are doing
+     *
+     * @author Olof Naessn
+     * @author Per Larsson.
      */
     class GCN_CORE_DECLSPEC Widget
     {
     public:
         /**
          * Constructor. Resets member variables. Noteable, a widget is not
-         * focusable as default, therefore, Widgets that are supposed to be
+         * focusable as default, therefore, widgets that are supposed to be
          * focusable should overide this default in their own constructor.
          */
         Widget();
-    
+
         /**
          * Default destructor.
          */
         virtual ~Widget();
 
         /**
-         * Draws the Widget. It is called by the parent widget when it is time
-         * for the Widget to draw itself. The graphics object is set up so
-         * that all drawing is relative to the Widget, i.e coordinate (0,0) is
-         * the top-left corner of the Widget. It is not possible to draw
-         * outside of a Widgets dimension.
+         * Draws the widget. It is called by the parent widget when it is time
+         * for the widget to draw itself. The graphics object is set up so
+         * that all drawing is relative to the widget, i.e coordinate (0,0) is
+         * the top-left corner of the widget. It is not possible to draw
+         * outside of a widgets dimension.
          *
          * @param graphics a Graphics object to draw with.
          */
         virtual void draw(Graphics* graphics) = 0;
-        
+
         /**
-         * Draws a the Widget border. A border is drawn around a Widget.
-         * The width and height of the border is therefore the Widgets
+         * Draws the widget border. A border is drawn around a widget.
+         * The width and height of the border is therefore the widgets
          * height+2*bordersize. Think of a painting that has a certain size,
-         * the border surrounds the painting. 
+         * the border surrounds the painting.
          *
          * @param graphics a Graphics object to draw with.
          */
         virtual void drawBorder(Graphics* graphics) { }
-        
+
         /**
-         * Called for all Widgets in the gui each time Gui::logic is called.
+         * Called for all widgets in the gui each time Gui::logic is called.
          * You can do logic stuff here like playing an animation.
          *
          * @see Gui
          */
         virtual void logic() { }
-    
+
         /**
-         * Gets the Widget parent container.
+         * Gets the widget parent container.
          *
-         * @return the Widget parent container. Returns NULL if the Widget
-         *         has no parent.         
+         * @return the widget parent container. Returns NULL if the widget
+         *         has no parent.
          */
-        virtual BasicContainer* getParent() const;
-    
+        virtual Widget* getParent() const;
+
         /**
-         * Sets the width of the Widget in pixels.
+         * Sets the width of the widget in pixels.
          *
-         * @param width the Widget width in pixels.
+         * @param width the widget width in pixels.
          */
         virtual void setWidth(int width);
 
         /**
-         * Gets the width of the Widget in pixels.
+         * Gets the width of the widget in pixels.
          *
-         * @return the Widget with in pixels.
+         * @return the widget with in pixels.
          */
         virtual int getWidth() const;
-    
+
         /**
-         * Sets the height of the Widget in pixels.
+         * Sets the height of the widget in pixels.
          *
-         * @param height the Widget height in pixels.
+         * @param height the widget height in pixels.
          */
         virtual void setHeight(int height);
 
         /**
-         * Gets the height of the Widget in pixels.
+         * Gets the height of the widget in pixels.
          *
-         * @return the Widget height in pixels.
+         * @return the widget height in pixels.
          */
         virtual int getHeight() const;
 
         /**
-         * Sets the size of the Widget.
+         * Sets the size of the widget.
          *
          * @param width the width.
          * @param height the height.
          */
         virtual void setSize(int width, int height);
-        
+
         /**
-         * Set the Widget x coordinate. It is relateive to it's parent.
+         * Set the widget x coordinate. It is relateive to it's parent.
          *
-         * @param x the Widget x coordinate.
+         * @param x the widget x coordinate.
          */
         virtual void setX(int x);
 
         /**
-         * Gets the Widget x coordinate. It is relative to it's parent.
+         * Gets the widget x coordinate. It is relative to it's parent.
          *
-         * @return the Widget x coordinate.
+         * @return the widget x coordinate.
          */
         virtual int getX() const;
 
         /**
-         * Set the Widget y coordinate. It is relative to it's parent.
+         * Set the widget y coordinate. It is relative to it's parent.
          *
-         * @param y the Widget y coordinate. 
+         * @param y the widget y coordinate.
          */
         virtual void setY(int y);
 
         /**
-         * Gets the Widget y coordinate. It is relative to it's parent.
+         * Gets the widget y coordinate. It is relative to it's parent.
          *
-         * @return the Widget y coordinate.
+         * @return the widget y coordinate.
          */
         virtual int getY() const;
 
         /**
-         * Sets the Widget position. It is relative to it's parent.
+         * Sets the widget position. It is relative to it's parent.
          *
-         * @param x the Widget x coordinate.
-         * @param y the Widgets y coordinate.
+         * @param x the widget x coordinate.
+         * @param y the widgets y coordinate.
          */
         virtual void setPosition(int x, int y);
-    
+
         /**
-         * Sets the dimension of the Widget. It is relative to it's parent.
+         * Sets the dimension of the widget. It is relative to it's parent.
          *
-         * @param dimension the Widget dimension.
+         * @param dimension the widget dimension.
          */
         virtual void setDimension(const Rectangle& dimension);
 
         /**
          * Sets the size of the border, or the width if you so like. The size
-         * is the number of pixels that the border extends outside the Widget.
-         * Border size = 0 means no border.         
+         * is the number of pixels that the border extends outside the widget.
+         * Border size = 0 means no border.
          *
          * @param borderSize the size of the border.
-         * @see drawBorder         
+         * @see drawBorder
         */
         virtual void setBorderSize(unsigned int borderSize);
 
         /**
          * Gets the size of the border, or the width if you so like. The size
-         * is the number of pixels that the border extends outside the Widget.
-         * Border size = 0 means no border.                      
+         * is the number of pixels that the border extends outside the widget.
+         * Border size = 0 means no border.
          *
          * @return the size of the border.
-         * @see drawBorder         
+         * @see drawBorder
          */
         virtual unsigned int getBorderSize() const;
-        
+
         /**
-         * Gets the dimension of the Widget. It is relative to it's parent.
+         * Gets the dimension of the widget. It is relative to it's parent.
          *
          * @return the widget dimension.
          */
         virtual const Rectangle& getDimension() const;
 
         /**
-         * Sets a Widgets focusability.
+         * Sets a widgets focusability.
          *
-         * @param focusable true if the Widget should be focusable.
+         * @param focusable true if the widget should be focusable.
          */
         virtual void setFocusable(bool focusable);
 
         /**
-         * Checks whether the Widget is focusable.
+         * Checks whether the widget is focusable.
          *
          * @return true if the widget is focusable.
          */
         virtual bool isFocusable() const;
-        
+
         /**
-         * Checks if the Widget has focus.
+         * Checks if the widget is focused.
          *
          * @return true if the widget currently has focus.
          */
-        virtual bool hasFocus() const;
+        virtual bool isFocused() const;
 
         /**
-         * Sets the Widget to be disabled or enabled. A disabled
-         * Widget will never recieve mouse or key input.
+         * Sets the widget to be disabled or enabled. A disabled
+         * widget will never recieve mouse or key input.
          *
-         * @param disabled true if Widget is enabled.
+         * @param enabled true if widget is enabled.
          */
         virtual void setEnabled(bool enabled);
 
         /**
-         * Checks if a Widget is disabled or not.
+         * Checks if a widget is disabled or not.
          *
-         * @return true if the Widget should be enabled.
+         * @return true if the widget should be enabled.
          */
-        virtual bool isEnabled() const; 
-        
+        virtual bool isEnabled() const;
+
         /**
-         * Called if the Widget looses focus.
-         */
-        virtual void lostFocus() { };
-    
-        /**
-         * Called if the Widget recieves focus.       
-         */
-        virtual void gotFocus() { };
-    
-        /**
-         * Checks if the Widget has the mouse.
+         * Sets the widget to be visible.
          *
-         * @return true if the Widget has the mouse.
-         */
-        virtual bool hasMouse() const;
-    
-        /**
-         * Sets the Widget to be visible.
-         *
-         * @param visible true if the Widget should be visiable.
+         * @param visible true if the widget should be visiable.
          */
         virtual void setVisible(bool visible);
 
         /**
-         * Checks if the Widget is visible.
+         * Checks if the widget is visible.
          *
-         * @return true if the Widget is visible.
+         * @return true if the widget is visible.
          */
         virtual bool isVisible() const;
 
         /**
          * Sets the base color. The base color is the background
-         * color for many Widgets like the Button and Contianer Widgets.
+         * color for many widgets like the Button and Contianer widgets.
          *
-         * @param color the baseground Color.
+         * @param color the baseground color.
          */
         virtual void setBaseColor(const Color& color);
 
         /**
          * Gets the base color.
          *
-         * @return the foreground Color.
+         * @return the foreground color.
          */
         virtual const Color& getBaseColor() const;
-    
+
         /**
          * Sets the foreground color.
          *
-         * @param color the foreground Color.
+         * @param color the foreground color.
          */
         virtual void setForegroundColor(const Color& color);
 
         /**
          * Gets the foreground color.
          *
-         * @return the foreground Color.
+         * @return the foreground color.
          */
         virtual const Color& getForegroundColor() const;
-    
+
         /**
          * Sets the background color.
          *
@@ -354,63 +336,37 @@ namespace gcn
         /**
          * Gets the background color.
          *
-         * @return the background Color.
+         * @return the background color.
          */
-        virtual const Color& getBackgroundColor() const;    
-            
-        /**
-         * Called when a Widget recieves a MouseInput.
-         *
-         * WARNING: This function is used internally to handle all mouse
-         *          messages. Don't call or overload it unless you know what
-         *          you are doing.
-         *
-         * @param mouseInput the MouseInput message.
-         */
-        virtual void _mouseInputMessage(const MouseInput& mouseInput);
+        virtual const Color& getBackgroundColor() const;
 
         /**
-         * Called when a Widget recieves a KeyInput.
+         * Sets the selection color.
          *
-         * WARNING: This function is used internally to handle all key
-         *          messages. Don't call or overload it unless you know what
-         *          you are doing.
-         *
-         * @param keyInput the KeyInput message.
+         * @param color the selection color.
          */
-        virtual void _keyInputMessage(const KeyInput& keyInput);
+        virtual void setSelectionColor(const Color& color);
 
         /**
-         * Called when the mouse enters the Widget area.
+         * Gets the selection color.
          *
-         * WARNING: This function is used internally to handle mouse in
-         *          messages. Don't call or overload this function unless
-         *          you know what you are doing.
+         * @return the selection color.
          */
-        virtual void _mouseInMessage();
-
+        virtual const Color& getSelectionColor() const;
+        
         /**
-         * Called when the mouse leaves the Widget area.
-         *
-         * WARNING: This function is used internally be to handle mouse
-         *          out messages. Don't call or overload this function
-         *          unless you know what you are doing.
-         */
-        virtual void _mouseOutMessage();
-    
-        /**
-         * Requests focus for the Widget. A Widget will only recieve focus
+         * Requests focus for the widget. A widget will only recieve focus
          * if it is focusable.
          */
         virtual void requestFocus();
 
         /**
-         * Requests a move to the top in the parent Widget.
+         * Requests a move to the top in the parent widget.
          */
         virtual void requestMoveToTop();
 
         /**
-         * Requests a move to the bottom in the parent Widget.
+         * Requests a move to the bottom in the parent widget.
          */
         virtual void requestMoveToBottom();
 
@@ -437,89 +393,123 @@ namespace gcn
         virtual FocusHandler* _getFocusHandler();
 
         /**
-         * Adds an ActionListener to the Widget. When an action is triggered
-         * by the Widget, the action function in all the Widget's
+         * Adds an ActionListener to the widget. When an action is triggered
+         * by the widget, the action function in all the widget's
          * ActionListeners will be called.
          *
          * @param actionListener the ActionListener to add.
          */
         virtual void addActionListener(ActionListener* actionListener);
-    
+
         /**
-         * Removes an added ActionListener from the Widget.
+         * Removes an added ActionListener from the widget.
          *
          * @param actionListener the ActionListener to remove.
          */
-        virtual    void removeActionListener(ActionListener* actionListener);
+        virtual void removeActionListener(ActionListener* actionListener);
 
         /**
-         * Adds a MouseListener to the Widget. When a mouse message is
-         * recieved, it will be sent to the Widget's MouseListeners.
+         * Adds a DeathListener to the widget. When the widget dies
+         * the death function in all the widget's DeathListeners will be called.
+         *
+         * @param actionListener the DeathListener to add.
+         */
+        virtual void addDeathListener(DeathListener* deathListener);
+
+        /**
+         * Removes an added DeathListener from the widget.
+         *
+         * @param deathListener the DeathListener to remove.
+         */
+        virtual void removeDeathListener(DeathListener* deathListener);
+
+        /**
+         * Adds a MouseListener to the widget. When a mouse message is
+         * recieved, it will be sent to the widget's MouseListeners.
          *
          * @param mouseListener the MouseListener to add.
          */
-        virtual    void addMouseListener(MouseListener* mouseListener);
-    
+        virtual void addMouseListener(MouseListener* mouseListener);
+
         /**
-         * Removes an added MouseListener from the Widget.
+         * Removes an added MouseListener from the widget.
          *
          * @param mouseListener the MouseListener to remove.
-         */    
-        virtual    void removeMouseListener(MouseListener* mouseListener);
+         */
+        virtual void removeMouseListener(MouseListener* mouseListener);
 
         /**
-         * Adds a KeyListener to the Widget. When a key message is recieved,
-         * it will be sent to the Widget's KeyListeners.
+         * Adds a KeyListener to the widget. When a key message is recieved,
+         * it will be sent to the widget's KeyListeners.
          *
          * @param keyListener the KeyListener to add.
-         */    
-        virtual    void addKeyListener(KeyListener* keyListener);
+         */
+        virtual void addKeyListener(KeyListener* keyListener);
 
         /**
-         * Removes an added KeyListener from the Widget.
+         * Removes an added KeyListener from the widget.
          *
          * @param keyListener the KeyListener to remove.
-         */    
-        virtual    void removeKeyListener(KeyListener* keyListener);
-
-        /**
-         * Sets the event identifier of the Widget. The event identifier is
-         * used to be able to identify which Widget generated an action when
-         * an action has occured.
-         *
-         * NOTE: An event identifier should not be used to identify a certain
-         *       Widget but rather a certain event in your application. Several
-         *       Widgets can have the same event identifer.
-         * 
-         * @param eventId the event identifier.
-         */    
-        virtual    void setEventId(const std::string& eventId);
-
-        /**
-         * Gets the event identifier.
-         *
-         * @return the event identifier.
          */
-        virtual const std::string& getEventId() const;
+        virtual void removeKeyListener(KeyListener* keyListener);
+
+         /**
+         * Adds a FocusListener to the widget. When a focus event is recieved,
+         * it will be sent to the widget's FocusListeners.
+         *
+         * @param focusListener the FocusListener to add.
+         * @author Olof Naessn
+         * @since 0.7.0
+         */
+        virtual void addFocusListener(FocusListener* focusListener);
 
         /**
-         * Gets the absolute position on the screen for the Widget.
+         * Removes an added FocusListener from the widget.
+         *
+         * @param focusListener the FocusListener to remove.
+         * @author Olof Naessn
+         * @since 0.7.0
+         */
+        virtual void removeFocusListener(FocusListener* FocusListener);
+
+        /**
+         * Sets the action event identifier of the widget. The identifier is
+         * used to be able to identify which action has occured.
+         *
+         * NOTE: An action event identifier should not be used to identify a
+         *       certain widget but rather a certain event in your application.
+         *       Several widgets can have the same action event identifer.
+         *
+         * @param actionEventId the action event identifier.
+         * @since 0.6.0
+         */
+        virtual void setActionEventId(const std::string& actionEventId);
+
+        /**
+         * Gets the action event identifier.
+         *
+         * @return the action event identifier.
+         */
+        virtual const std::string& getActionEventId() const;
+
+        /**
+         * Gets the absolute position on the screen for the widget.
          *
          * @param x absolute x coordinate will be stored in this parameter.
          * @param y absolute y coordinate will be stored in this parameter.
-         */    
-        virtual    void getAbsolutePosition(int& x, int& y) const;
-    
+         */
+        virtual void getAbsolutePosition(int& x, int& y) const;
+
         /**
-         * Sets the parent of the Widget. A parent must be a BasicContainer.
+         * Sets the parent of the widget. A parent must be a BasicContainer.
          *
          * WARNING: This function is used internally and should not
          *          be called or overloaded unless you know what you
          *          are doing.
          *
-         * @param parent the parent BasicContainer.. 
-         */    
-        virtual    void _setParent(BasicContainer* parent);
+         * @param parent the parent BasicContainer..
+         */
+        virtual void _setParent(Widget* parent);
 
         /**
          * Gets the font used. If no font has been set, the global font will
@@ -530,14 +520,14 @@ namespace gcn
          * @return the used Font.
          */
         Font *getFont() const;
-    
+
         /**
-         * Sets the global font to be used by default for all Widgets.
+         * Sets the global font to be used by default for all widgets.
          *
          * @param font the global Font.
          */
         static void setGlobalFont(Font* font);
-    
+
         /**
          * Sets the font. If font is NULL, the global font will be used.
          *
@@ -547,32 +537,32 @@ namespace gcn
 
         /**
          * Called when the font has changed. If the change is global,
-         * this function will only be called if the Widget don't have a
+         * this function will only be called if the widget don't have a
          * font already set.
          */
         virtual void fontChanged() { }
 
         /**
-         * Checks whether a Widget exists or not, that is if it still exists
+         * Checks whether a widget exists or not, that is if it still exists
          * an instance of the object.
          *
-         * @param widget the Widget to check.
+         * @param widget the widget to check.
          */
         static bool widgetExists(const Widget* widget);
 
         /**
          * Check if tab in is enabled. Tab in means that you can set focus
-         * to this Widget by pressing the tab button. If tab in is disabled
+         * to this widget by pressing the tab button. If tab in is disabled
          * then the FocusHandler will skip this widget and focus the next
          * in its focus order.
          *
          * @return true if tab in is enabled.
          */
         virtual bool isTabInEnabled() const;
-        
+
         /**
          * Sets tab in enabled. Tab in means that you can set focus
-         * to this Widget by pressing the tab button. If tab in is disabled
+         * to this widget by pressing the tab button. If tab in is disabled
          * then the FocusHandler will skip this widget and focus the next
          * in its focus order.
          *
@@ -582,95 +572,215 @@ namespace gcn
 
         /**
          * Checks if tab out is enabled. Tab out means that you can lose
-         * focus to this Widget by pressing the tab button. If tab out is
+         * focus to this widget by pressing the tab button. If tab out is
          * disabled then the FocusHandler ignores tabbing and focus will
-         * stay with this Widget.
+         * stay with this widget.
          *
          * @return true if tab out is enabled.
          */
         virtual bool isTabOutEnabled() const;
-        
+
         /**
          * Sets tab out enabled. Tab out means that you can lose
-         * focus to this Widget by pressing the tab button. If tab out is
+         * focus to this widget by pressing the tab button. If tab out is
          * disabled then the FocusHandler ignores tabbing and focus will
-         * stay with this Widget.
+         * stay with this widget.
          *
          * @param enabled true if tab out should be enabled.
          */
         virtual void setTabOutEnabled(bool enabled);
 
-        /**
-         * Checks if the Widget is dragged. Dragged means that the mouse
-         * button has been pressed down over the Widget and the mouse has
-         * been moved.
-         *
-         * @return true if the widget is dragged.
-         */
-        virtual bool isDragged() const;
-
         /**
          * Requests modal focus. When a widget has modal focus, only that
-         * Widget and it's children may recieve input. If some other Widget
-         * already has modal focus, an exception will be thrown.
+         * widget and it's children may recieve input.
          *
-         * @throws Exception if another Widget already has modal focus.
+         * @throws Exception if another widget already has modal focus.
          */
         virtual void requestModalFocus();
 
+        /**
+         * Requests modal mouse input focus. When a widget has modal input focus
+         * that widget will be the only widget receiving input even if the input
+         * occurs outside of the widget and no matter what the input is.
+         *
+         * @throws Exception if another widget already has modal focus.
+         * @since 0.6.0
+         */
+        virtual void requestModalMouseInputFocus();
+
         /**
          * Releases modal focus. Modal focus will only be released if the
-         * Widget has the modal focus.
+         * widget has modal focus.
          */
         virtual void releaseModalFocus();
-        
+
         /**
-         * Checks if the Widget or it's parent has modal focus.
+         * Releases modal mouse input focus. Modal mouse input focus will only
+         * be released if the widget has modal mouse input focus.
+         *
+         * @since 0.6.0
+         */
+        virtual void releaseModalMouseInputFocus();
+
+        /**
+         * Checks if the widget or it's parent has modal focus.
          */
         virtual bool hasModalFocus() const;
 
-        
+        /**
+         * Checks if the widget or it's parent has modal mouse input focus.
+         *
+         * @since 0.6.0
+         */
+        virtual bool hasModalMouseInputFocus() const;
+
+        /**
+         * Gets a widget from a certain position in the widget.
+         * This function is used to decide which gets mouse input,
+         * thus it can be overloaded to change that behaviour.
+         *
+         * NOTE: This always returns NULL if the widget is not
+         *       a container.
+         *
+         * @param x the x coordinate.
+         * @param y the y coordinate.
+         * @return the widget at the specified coodinate, or NULL
+         *         if no such widget exists.
+         * @since 0.6.0
+         */
+        virtual Widget *getWidgetAt(int x, int y);
+
+        /**
+         * Gets the mouse listeners of the widget.
+         *
+         * @return the mouse listeners of the widget.
+         * @since 0.6.0
+         */
+        virtual const std::list<MouseListener*>& _getMouseListeners();
+
+        /**
+         * Gets the key listeners of the widget.
+         *
+         * @return the key listeners of the widget.
+         * @since 0.6.0
+         */
+        virtual const std::list<KeyListener*>& _getKeyListeners();
+
+        /**
+         * Gets the focus listeners of the widget.
+         *
+         * @return the focus listeners of the widget.
+         * @since 0.7.0
+         */
+        virtual const std::list<FocusListener*>& _getFocusListeners();
+
+        /**
+         * Gets the subarea of the widget that the children occupy.
+         *
+         * @return the subarea as a Rectangle.
+         */
+        virtual Rectangle getChildrenArea();
+
+        /**
+         * Gets the internal FocusHandler used.
+         *
+         * @return the internalFocusHandler used. If no internal FocusHandler
+         *         is used, NULL will be returned.
+         */
+        virtual FocusHandler* _getInternalFocusHandler();
+
+        /**
+         * Sets the internal FocusHandler. An internal focushandler is
+         * needed if both a widget in the widget and the widget itself
+         * should be foucsed at the same time.
+         *
+         * @param focusHandler the FocusHandler to be used.
+         */
+        virtual void setInternalFocusHandler(FocusHandler* focusHandler);
+
+        /**
+         * Moves a widget to the top of this widget. The moved widget will be
+         * drawn above all other widgets in this widget.
+         *
+         * @param widget the widget to move.
+         */
+        virtual void moveToTop(Widget* widget) { };
+
+        /**
+         * Moves a widget in this widget to the bottom of this widget.
+         * The moved widget will be drawn below all other widgets in this widget.
+         *
+         * @param widget the widget to move.
+         */
+        virtual void moveToBottom(Widget* widget) { };
+
+        /**
+         * Focuses the next widget in the widget.
+         */
+        virtual void focusNext() { };
+
+        /**
+         * Focuses the previous widget in the widget.
+         */
+        virtual void focusPrevious() { };
+
+        /**
+         * Tries to show a specific part of a widget by moving it. Used if the
+         * widget should act as a container.
+         *
+         * @param widget the target widget.
+         * @param area the area to show.
+         */
+        virtual void showWidgetPart(Widget* widget, Rectangle area) { };
+
     protected:
         /**
-         * Generates an action to the Widget's ActionListeners.
+         * Generates an action to the widget's ActionListeners.
          */
         void generateAction();
-        
+
         typedef std::list<MouseListener*> MouseListenerList;
-        MouseListenerList mMouseListeners;
         typedef MouseListenerList::iterator MouseListenerIterator;
-    
+        MouseListenerList mMouseListeners;
+
+
         typedef std::list<KeyListener*> KeyListenerList;
         KeyListenerList mKeyListeners;
         typedef KeyListenerList::iterator KeyListenerIterator;
-        
+
         typedef std::list<ActionListener*> ActionListenerList;
         ActionListenerList mActionListeners;
         typedef ActionListenerList::iterator ActionListenerIterator;
-        
+
+        typedef std::list<DeathListener*> DeathListenerList;
+        DeathListenerList mDeathListeners;
+        typedef DeathListenerList::iterator DeathListenerIterator;
+
+        typedef std::list<FocusListener*> FocusListenerList;
+        FocusListenerList mFocusListeners;
+        typedef FocusListenerList::iterator FocusListenerIterator;
+
         Color mForegroundColor;
         Color mBackgroundColor;
         Color mBaseColor;
+        Color mSelectionColor;
         FocusHandler* mFocusHandler;
-        BasicContainer* mParent;
+        FocusHandler* mInternalFocusHandler;
+        Widget* mParent;
         Rectangle mDimension;
         unsigned int mBorderSize;
-        std::string mEventId;
-        int mClickTimeStamp;
-        int mClickCount;
-        int mClickButton;
-        bool mHasMouse;
+        std::string mActionEventId;
         bool mFocusable;
         bool mVisible;
         bool mTabIn;
         bool mTabOut;
         bool mEnabled;
-        
+
         Font* mCurrentFont;
         static DefaultFont mDefaultFont;
         static Font* mGlobalFont;
         static std::list<Widget*> mWidgets;
-    };  
+    };
 }
 
 #endif // end GCN_WIDGET_HPP
