@@ -113,7 +113,7 @@ namespace gcn
     {
         bool result = Graphics::pushClipArea(area);
 
-        ClipRectangle cr = mClipStack.top();
+        const ClipRectangle& cr = mClipStack.top();
 
         // Allegro won't let you set clip areas
         // that have zero width or height
@@ -126,9 +126,17 @@ namespace gcn
         {
             mClipNull = false;
 #if ALLEGRO_VERSION == 4 && ALLEGRO_SUB_VERSION == 0
-            set_clip(mTarget, cr.x, cr.y, cr.x + cr.width - 1, cr.y + cr.height - 1);
+            set_clip(mTarget,
+                     cr.x,
+                     cr.y,
+                     cr.x + cr.width - 1,
+                     cr.y + cr.height - 1);
 #else
-            set_clip_rect(mTarget, cr.x, cr.y, cr.x + cr.width - 1, cr.y + cr.height - 1);
+            set_clip_rect(mTarget,
+                          cr.x,
+                          cr.y,
+                          cr.x + cr.width - 1,
+                          cr.y + cr.height - 1);
 #endif
         }
 
@@ -144,7 +152,7 @@ namespace gcn
             return;
         }
 
-        ClipRectangle cr = mClipStack.top();
+        const ClipRectangle& cr = mClipStack.top();
 
         // Allegro won't let you set clip areas
         //that have zero width or height
@@ -157,25 +165,42 @@ namespace gcn
         {
             mClipNull = false;
 #if ALLEGRO_VERSION == 4 && ALLEGRO_SUB_VERSION == 0
-            set_clip(mTarget, cr.x, cr.y, cr.x + cr.width - 1, cr.y + cr.height - 1);
+            set_clip(mTarget,
+                     cr.x,
+                     cr.y,
+                     cr.x + cr.width - 1,
+                     cr.y + cr.height - 1);
 #else
-            set_clip_rect(mTarget, cr.x, cr.y, cr.x + cr.width - 1, cr.y + cr.height - 1);
+            set_clip_rect(mTarget,
+                          cr.x,
+                          cr.y,
+                          cr.x + cr.width - 1,
+                          cr.y + cr.height - 1);
 #endif
         }
     }
 
     void AllegroGraphics::drawImage(const Image* image,
-                                    int srcX, int srcY,
-                                    int dstX, int dstY,
-                                    int width, int height)
+                                    int srcX,
+                                    int srcY,
+                                    int dstX,
+                                    int dstY,
+                                    int width,
+                                    int height)
     {
         if (mClipNull)
         {
             return;
         }
 
-        dstX += mClipStack.top().xOffset;
-        dstY += mClipStack.top().yOffset;
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion "
+                                "outside of _beginDraw() and _endDraw()?");
+        }
+
+        const int xOffset = mClipStack.top().xOffset;
+        const int yOffset = mClipStack.top().yOffset;
 
         const AllegroImage* srcImage = dynamic_cast<const AllegroImage*>(image);
 
@@ -184,7 +209,14 @@ namespace gcn
             throw GCN_EXCEPTION("Trying to draw an image of unknown format, must be an AllegroImage.");
         }
 
-        masked_blit(srcImage->getBitmap(), mTarget, srcX, srcY, dstX, dstY, width, height);
+        masked_blit(srcImage->getBitmap(),
+                    mTarget,
+                    srcX,
+                    srcY,
+                    dstX + xOffset,
+                    dstY + yOffset,
+                    width,
+                    height);
     }
 
     void AllegroGraphics::drawPoint(int x, int y)
@@ -194,13 +226,19 @@ namespace gcn
             return;
         }
 
-        int xOffset = mClipStack.top().xOffset;
-        int yOffset = mClipStack.top().yOffset;
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion "
+                                "outside of _beginDraw() and _endDraw()?");
+        }
+
+        const int xOffset = mClipStack.top().xOffset;
+        const int yOffset = mClipStack.top().yOffset;
 
         putpixel(mTarget,
                  x + xOffset,
                  y + yOffset,
-                 mAlColor);
+                 mAllegroColor);
     }
 
     void AllegroGraphics::drawLine(int x1, int y1, int x2, int y2)
@@ -210,15 +248,21 @@ namespace gcn
             return;
         }
 
-        int xOffset = mClipStack.top().xOffset;
-        int yOffset = mClipStack.top().yOffset;
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion "
+                                "outside of _beginDraw() and _endDraw()?");
+        }
+
+        const int xOffset = mClipStack.top().xOffset;
+        const int yOffset = mClipStack.top().yOffset;
 
         line(mTarget,
              x1 + xOffset,
              y1 + yOffset,
              x2 + xOffset,
              y2 + yOffset,
-             mAlColor);
+             mAllegroColor);
     }
 
     void AllegroGraphics::drawRectangle(const Rectangle& rectangle)
@@ -228,15 +272,21 @@ namespace gcn
             return;
         }
 
-        int xOffset = mClipStack.top().xOffset;
-        int yOffset = mClipStack.top().yOffset;
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion " 
+                                "outside of _beginDraw() and _endDraw()?");
+        }
+
+        const int xOffset = mClipStack.top().xOffset;
+        const int yOffset = mClipStack.top().yOffset;
 
         rect(mTarget,
              rectangle.x + xOffset,
              rectangle.y + yOffset,
              rectangle.x + rectangle.width - 1 + xOffset,
              rectangle.y + rectangle.height - 1 + yOffset,
-             mAlColor);
+             mAllegroColor);
     }
 
     void AllegroGraphics::fillRectangle(const Rectangle& rectangle)
@@ -246,21 +296,27 @@ namespace gcn
             return;
         }
 
-        int xOffset = mClipStack.top().xOffset;
-        int yOffset = mClipStack.top().yOffset;
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion " 
+                                "outside of _beginDraw() and _endDraw()?");
+        }
+
+        const int xOffset = mClipStack.top().xOffset;
+        const int yOffset = mClipStack.top().yOffset;
 
         rectfill(mTarget,
                  rectangle.x + xOffset,
                  rectangle.y + yOffset,
                  rectangle.x + rectangle.width - 1 + xOffset,
                  rectangle.y + rectangle.height - 1 + yOffset,
-                 mAlColor);
+                 mAllegroColor);
     }
 
     void AllegroGraphics::setColor(const Color& color)
     {
         mColor = color;
-        mAlColor = makecol(color.r, color.g, color.b);
+        mAllegroColor = makecol(color.r, color.g, color.b);
 
         if (color.a != 255)
         {
@@ -273,8 +329,28 @@ namespace gcn
         }
     }
 
-    const Color& AllegroGraphics::getColor()
+    const Color& AllegroGraphics::getColor() const
     {
         return mColor;
     }
+
+    int AllegroGraphics::getAllegroColor() const
+    {
+        return mAllegroColor;
+    }
+
+    void AllegroGraphics::drawBitmap(BITMAP* bitmap, int dstX, int dstY)
+    {
+        const int xOffset = mClipStack.top().xOffset;
+        const int yOffset = mClipStack.top().yOffset;
+
+        masked_blit(bitmap,
+                    mTarget,
+                    0,
+                    0,
+                    dstX + xOffset,
+                    dstY + yOffset,
+                    bitmap->w,
+                    bitmap->h);
+    }
 }
