@@ -127,8 +127,13 @@ namespace gcn
         glMatrixMode(GL_PROJECTION);
         glPushMatrix();
         glLoadIdentity();
-
-        glOrtho(0.0, (double)mWidth, (double)mHeight, 0.0, -1.0, 1.0);
+        
+        glOrtho(0.0,
+                (double)mWidth,
+                (double)mHeight,
+                0.0,
+                -1.0,
+                1.0);
 
         glDisable(GL_LIGHTING);
         glDisable(GL_CULL_FACE);
@@ -195,8 +200,12 @@ namespace gcn
         mHeight = height;
     }
 
-    void OpenGLGraphics::drawImage(const Image* image, int srcX, int srcY,
-                                   int dstX, int dstY, int width,
+    void OpenGLGraphics::drawImage(const Image* image,
+                                   int srcX,
+                                   int srcY,
+                                   int dstX,
+                                   int dstY,
+                                   int width,
                                    int height)
     {
 		const OpenGLImage* srcImage = dynamic_cast<const OpenGLImage*>(image);
@@ -206,8 +215,15 @@ namespace gcn
             throw GCN_EXCEPTION("Trying to draw an image of unknown format, must be an OpenGLImage.");
         }
 
-        dstX += mClipStack.top().xOffset;
-        dstY += mClipStack.top().yOffset;
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
+        dstX += top.xOffset;
+        dstY += top.yOffset;
 
         // Find OpenGL texture coordinates
         float texX1 = srcX / (float)srcImage->getTextureWidth();
@@ -250,56 +266,94 @@ namespace gcn
 
     void OpenGLGraphics::drawPoint(int x, int y)
     {
-        x += mClipStack.top().xOffset;
-        y += mClipStack.top().yOffset;
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
+        x += top.xOffset;
+        y += top.yOffset;
 
         glBegin(GL_POINTS);
-        glVertex3i(x, y, 0);
+        glVertex2i(x, y);
         glEnd();
     }
 
     void OpenGLGraphics::drawLine(int x1, int y1, int x2, int y2)
     {
-        x1 += mClipStack.top().xOffset;
-        y1 += mClipStack.top().yOffset;
-        x2 += mClipStack.top().xOffset;
-        y2 += mClipStack.top().yOffset;
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
+        x1 += top.xOffset;
+        y1 += top.yOffset;
+        x2 += top.xOffset;
+        y2 += top.yOffset;
 
         glBegin(GL_LINES);
-        glVertex3f(x1+0.5f, y1+0.5f, 0);
-        glVertex3f(x2+0.5f, y2+0.5f, 0);
+        glVertex2f(x1 + 0.375f,
+                   y1 + 0.375f);
+        glVertex2f(x2 + 1.0f - 0.375f,
+                   y2 + 1.0f - 0.375f);
         glEnd();
 
         glBegin(GL_POINTS);
-        glVertex3f(x2+0.5f, y2+0.5f, 0);
+        glVertex2f(x2 + 1.0f - 0.375f,
+                   y2 + 1.0f - 0.375f);
         glEnd();
+
+        glBegin(GL_POINTS);
+        glVertex2f(x1 + 0.375f,
+                   y1 + 0.375f);
+        glEnd();
+
+        
     }
 
     void OpenGLGraphics::drawRectangle(const Rectangle& rectangle)
     {
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+        
         glBegin(GL_LINE_LOOP);
-        glVertex3f(rectangle.x + mClipStack.top().xOffset + 0.5f,
-                   rectangle.y + mClipStack.top().yOffset + 0.5f, 0);
-        glVertex3f(rectangle.x + rectangle.width - 0.5f + mClipStack.top().xOffset,
-                   rectangle.y + mClipStack.top().yOffset + 0.5f, 0);
-        glVertex3f(rectangle.x + rectangle.width - 0.5f + mClipStack.top().xOffset,
-                   rectangle.y + rectangle.height + mClipStack.top().yOffset - 0.5f, 0);
-        glVertex3f(rectangle.x + mClipStack.top().xOffset + 0.5f,
-                   rectangle.y + rectangle.height + mClipStack.top().yOffset - 0.5f, 0);
+        glVertex2f(rectangle.x + top.xOffset,
+                   rectangle.y + top.yOffset);
+        glVertex2f(rectangle.x + rectangle.width + top.xOffset - 1.0f,
+                   rectangle.y + top.yOffset + 0.375f);
+        glVertex2f(rectangle.x + rectangle.width + top.xOffset - 1.0f,
+                   rectangle.y + rectangle.height + top.yOffset);
+        glVertex2f(rectangle.x + top.xOffset,
+                   rectangle.y + rectangle.height + top.yOffset);
         glEnd();
     }
 
     void OpenGLGraphics::fillRectangle(const Rectangle& rectangle)
     {
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
         glBegin(GL_QUADS);
-        glVertex3i(rectangle.x + mClipStack.top().xOffset,
-                   rectangle.y + mClipStack.top().yOffset, 0);
-        glVertex3i(rectangle.x + rectangle.width + mClipStack.top().xOffset,
-                   rectangle.y + mClipStack.top().yOffset, 0);
-        glVertex3i(rectangle.x + rectangle.width + mClipStack.top().xOffset,
-                   rectangle.y + rectangle.height + mClipStack.top().yOffset, 0);
-        glVertex3i(rectangle.x + mClipStack.top().xOffset,
-                   rectangle.y + rectangle.height + mClipStack.top().yOffset, 0);
+        glVertex2i(rectangle.x + top.xOffset,
+                   rectangle.y + top.yOffset);
+        glVertex2i(rectangle.x + rectangle.width + top.xOffset,
+                   rectangle.y + top.yOffset);
+        glVertex2i(rectangle.x + rectangle.width + top.xOffset,
+                   rectangle.y + rectangle.height + top.yOffset);
+        glVertex2i(rectangle.x + top.xOffset,
+                   rectangle.y + rectangle.height + top.yOffset);
         glEnd();
     }
 
@@ -316,7 +370,7 @@ namespace gcn
         }
     }
 
-    const Color& OpenGLGraphics::getColor()
+    const Color& OpenGLGraphics::getColor() const
     {
         return mColor;
     }
