@@ -1,12 +1,14 @@
-/*      _______   __   __   __   ______   __   __   _______   __   __                 
- *     / _____/\ / /\ / /\ / /\ / ____/\ / /\ / /\ / ___  /\ /  |\/ /\                
- *    / /\____\// / // / // / // /\___\// /_// / // /\_/ / // , |/ / /                 
- *   / / /__   / / // / // / // / /    / ___  / // ___  / // /| ' / /                  
- *  / /_// /\ / /_// / // / // /_/_   / / // / // /\_/ / // / |  / /                   
- * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /                    
- * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/                      
+/*      _______   __   __   __   ______   __   __   _______   __   __
+ *     / _____/\ / /\ / /\ / /\ / ____/\ / /\ / /\ / ___  /\ /  |\/ /\
+ *    / /\____\// / // / // / // /\___\// /_// / // /\_/ / // , |/ / /
+ *   / / /__   / / // / // / // / /    / ___  / // ___  / // /| ' / /
+ *  / /_// /\ / /_// / // / // /_/_   / / // / // /\_/ / // / |  / /
+ * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /
+ * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/
  *
- * Copyright (c) 2004, 2005 darkbits                        Js_./
+ * Copyright (c) 2004, 2005, 2006, 2007 Olof Naessn and Per Larsson
+ *
+ *                                                         Js_./
  * Per Larsson a.k.a finalman                          _RqZ{a<^_aa
  * Olof Naessn a.k.a jansem/yakslem                _asww7!uY`>  )\a//
  *                                                 _Qhm`] _f "'c  1!5m
@@ -53,12 +55,15 @@
  */
 
 /*
- * For comments regarding functions please see the header file. 
+ * For comments regarding functions please see the header file.
  */
 
+#include "guichan/sdl/sdlgraphics.hpp"
+
 #include "guichan/exception.hpp"
 #include "guichan/font.hpp"
-#include "guichan/sdl/sdlgraphics.hpp"
+#include "guichan/image.hpp"
+#include "guichan/sdl/sdlimage.hpp"
 #include "guichan/sdl/sdlpixel.hpp"
 
 // For some reason an old version of MSVC did not like std::abs,
@@ -74,7 +79,7 @@ namespace gcn
     {
         mAlpha = false;
     }
-    
+
     void SDLGraphics::_beginDraw()
     {
         Rectangle area;
@@ -82,64 +87,74 @@ namespace gcn
         area.y = 0;
         area.width = mTarget->w;
         area.height = mTarget->h;
-        pushClipArea(area);        
+        pushClipArea(area);
     }
 
     void SDLGraphics::_endDraw()
     {
         popClipArea();
     }
-    
+
     void SDLGraphics::setTarget(SDL_Surface* target)
     {
-        mTarget = target;        
+        mTarget = target;
     }
 
     bool SDLGraphics::pushClipArea(Rectangle area)
     {
         SDL_Rect rect;
         bool result = Graphics::pushClipArea(area);
-        
-        ClipRectangle carea = mClipStack.top();
+
+        const ClipRectangle& carea = mClipStack.top();
         rect.x = carea.x;
         rect.y = carea.y;
         rect.w = carea.width;
         rect.h = carea.height;
-    
+
         SDL_SetClipRect(mTarget, &rect);
 
-        return result;    
+        return result;
     }
 
     void SDLGraphics::popClipArea()
     {
-        SDL_Rect rect;
         Graphics::popClipArea();
 
         if (mClipStack.empty())
         {
             return;
         }
-        
-        ClipRectangle carea = mClipStack.top();
+
+        const ClipRectangle& carea = mClipStack.top();
+        SDL_Rect rect;
         rect.x = carea.x;
         rect.y = carea.y;
         rect.w = carea.width;
         rect.h = carea.height;
-    
-        SDL_SetClipRect(mTarget, &rect);    
+
+        SDL_SetClipRect(mTarget, &rect);
     }
-  
+
     SDL_Surface* SDLGraphics::getTarget() const
     {
         return mTarget;
     }
-  
-    void SDLGraphics::drawImage(const Image* image, int srcX,
-                                int srcY, int dstX, int dstY,
-                                int width, int height)
+
+    void SDLGraphics::drawImage(const Image* image,
+                                int srcX,
+                                int srcY,
+                                int dstX,
+                                int dstY,
+                                int width,
+                                int height)
     {
-        ClipRectangle top = mClipStack.top();
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
         SDL_Rect src;
         SDL_Rect dst;
         src.x = srcX;
@@ -149,16 +164,26 @@ namespace gcn
         dst.x = dstX + top.xOffset;
         dst.y = dstY + top.yOffset;
 
-        SDL_Surface* srcImage = (SDL_Surface*)image->_getData();
-    
-        SDL_BlitSurface(srcImage, &src, mTarget, &dst);    
+        const SDLImage* srcImage = dynamic_cast<const SDLImage*>(image);
+
+        if (srcImage == NULL)
+        {
+            throw GCN_EXCEPTION("Trying to draw an image of unknown format, must be an SDLImage.");
+        }
+
+        SDL_BlitSurface(srcImage->getSurface(), &src, mTarget, &dst);
     }
 
     void SDLGraphics::fillRectangle(const Rectangle& rectangle)
-    {    
-        Rectangle area = rectangle;
-        ClipRectangle top = mClipStack.top(); 
+    {
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
 
+        const ClipRectangle& top = mClipStack.top();
+        
+        Rectangle area = rectangle;
         area.x += top.xOffset;
         area.y += top.yOffset;
 
@@ -175,6 +200,7 @@ namespace gcn
             int y2 = area.y + area.height < top.y + top.height ? area.y + area.height : top.y + top.height;
             int x, y;
 
+            SDL_LockSurface(mTarget);
             for (y = y1; y < y2; y++)
             {
                 for (x = x1; x < x2; x++)
@@ -182,7 +208,8 @@ namespace gcn
                     SDLputPixelAlpha(mTarget, x, y, mColor);
                 }
             }
-            
+            SDL_UnlockSurface(mTarget);
+
         }
         else
         {
@@ -191,15 +218,25 @@ namespace gcn
             rect.y = area.y;
             rect.w = area.width;
             rect.h = area.height;
-    
-            Uint32 color = SDL_MapRGBA(mTarget->format, mColor.r, mColor.g, mColor.b, mColor.a);
+
+            Uint32 color = SDL_MapRGBA(mTarget->format,
+                                       mColor.r,
+                                       mColor.g,
+                                       mColor.b,
+                                       mColor.a);
             SDL_FillRect(mTarget, &rect, color);
-        }        
+        }
     }
 
     void SDLGraphics::drawPoint(int x, int y)
     {
-        ClipRectangle top = mClipStack.top();
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+        
         x += top.xOffset;
         y += top.yOffset;
 
@@ -211,21 +248,29 @@ namespace gcn
             SDLputPixelAlpha(mTarget, x, y, mColor);
         }
         else
-        {            
+        {
             SDLputPixel(mTarget, x, y, mColor);
-        }    
+        }
     }
 
     void SDLGraphics::drawHLine(int x1, int y, int x2)
     {
-        ClipRectangle top = mClipStack.top();
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
         x1 += top.xOffset;
         y += top.yOffset;
         x2 += top.xOffset;
 
         if (y < top.y || y >= top.y + top.height)
+        {
             return;
-    
+        }
+
         if (x1 > x2)
         {
             x1 ^= x2;
@@ -239,6 +284,7 @@ namespace gcn
             {
                 return;
             }
+
             x1 = top.x;
         }
 
@@ -247,91 +293,104 @@ namespace gcn
             if (top.x + top.width <= x1)
             {
                 return;
-            }      
+            }
+            
             x2 = top.x + top.width -1;
         }
-    
+
         int bpp = mTarget->format->BytesPerPixel;
-    
+
         SDL_LockSurface(mTarget);
 
         Uint8 *p = (Uint8 *)mTarget->pixels + y * mTarget->pitch + x1 * bpp;
-    
-        Uint32 pixel = SDL_MapRGB(mTarget->format, mColor.r, mColor.g, mColor.b);
-    
-        switch(bpp) {
-          case 1:
-          {
-              for (;x1 <= x2; ++x1)
-              { 
-                  *(p++) = pixel;
-              }
-          } break;
-      
-          case 2:
-          {
-              Uint16* q = (Uint16*)p;
-              for (;x1 <= x2; ++x1)
-              {
-                  *(q++) = pixel;
-              }
-          } break;
-        
-          case 3:  
-          {
-              if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
-                  for (;x1 <= x2; ++x1)
-                  {
-                      p[0] = (pixel >> 16) & 0xff;
-                      p[1] = (pixel >> 8) & 0xff;
-                      p[2] = pixel & 0xff;
-                      p += 3;
-                  }
-              }
-              else
-              {
-                  for (;x1 <= x2; ++x1)
-                  {
-                      p[0] = pixel & 0xff;
-                      p[1] = (pixel >> 8) & 0xff;
-                      p[2] = (pixel >> 16) & 0xff;
-                      p += 3;
-                  }
-              } 
-          } break;
-  
-          case 4:
-          {
-              Uint32* q = (Uint32*)p;
-              for (;x1 <= x2; ++x1)
-              {
-                  if (mAlpha)
-                  {
-                      *q = SDLAlpha32(pixel,*q,mColor.a);
-                      q++;
-                  }
-                  else
-                  {
-                      *(q++) = pixel;
-                  }
-              }
-          } break;
 
+        Uint32 pixel = SDL_MapRGB(mTarget->format,
+                                  mColor.r,
+                                  mColor.g,
+                                  mColor.b);
+        Uint16* q;
+        switch(bpp)
+        {
+            case 1:
+                for (;x1 <= x2; ++x1)
+                {
+                    *(p++) = pixel;
+                }
+                break;
+                
+            case 2:
+            {
+                Uint16* q = (Uint16*)p;
+                for (;x1 <= x2; ++x1)
+                {
+                    *(q++) = pixel;
+                }
+                break;
+            }
+            case 3:
+                if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
+                {
+                    for (;x1 <= x2; ++x1)
+                    {
+                        p[0] = (pixel >> 16) & 0xff;
+                        p[1] = (pixel >> 8) & 0xff;
+                        p[2] = pixel & 0xff;
+                        p += 3;
+                    }
+                }
+                else
+                {
+                    for (;x1 <= x2; ++x1)
+                    {
+                        p[0] = pixel & 0xff;
+                        p[1] = (pixel >> 8) & 0xff;
+                        p[2] = (pixel >> 16) & 0xff;
+                        p += 3;
+                    }
+                }
+                break;
+
+            case 4:  
+            {          
+                Uint32* q = (Uint32*)p;
+                for (;x1 <= x2; ++x1)
+                {
+                    if (mAlpha)
+                    {
+                        *q = SDLAlpha32(pixel,*q,mColor.a);
+                        q++;
+                    }
+                    else
+                    {
+                        *(q++) = pixel;
+                    }
+                }
+                break;
+            }
+                
         } // end switch
-    
-        SDL_UnlockSurface(mTarget);    
+
+        SDL_UnlockSurface(mTarget);
     }
 
     void SDLGraphics::drawVLine(int x, int y1, int y2)
     {
-        ClipRectangle top = mClipStack.top();
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
         x += top.xOffset;
         y1 += top.yOffset;
         y2 += top.yOffset;
 
         if (x < top.x || x >= top.x + top.width)
+        {
             return;
-    
+        }
+        
         if (y1 > y2)
         {
             y1 ^= y2;
@@ -345,6 +404,7 @@ namespace gcn
             {
                 return;
             }
+
             y1 = top.y;
         }
 
@@ -353,40 +413,40 @@ namespace gcn
             if (top.y + top.height <= y1)
             {
                 return;
-            }      
+            }
+
             y2 = top.y + top.height - 1;
         }
-    
+
         int bpp = mTarget->format->BytesPerPixel;
-    
+
         SDL_LockSurface(mTarget);
-    
+
         Uint8 *p = (Uint8 *)mTarget->pixels + y1 * mTarget->pitch + x * bpp;
-    
+
         Uint32 pixel = SDL_MapRGB(mTarget->format, mColor.r, mColor.g, mColor.b);
-    
-        switch(bpp) {
+
+        switch(bpp)
+        {            
           case 1:
-          {
               for (;y1 <= y2; ++y1)
-              { 
+              {
                   *p = pixel;
                   p += mTarget->pitch;
               }
-          } break;
-      
+              break;
+
           case 2:
-          {
               for (;y1 <= y2; ++y1)
               {
                   *(Uint16*)p = pixel;
                   p += mTarget->pitch;
               }
-          } break;
-        
-          case 3:  
-          {
-              if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
+              break;
+
+          case 3:
+              if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
+              {
                   for (;y1 <= y2; ++y1)
                   {
                       p[0] = (pixel >> 16) & 0xff;
@@ -404,11 +464,10 @@ namespace gcn
                       p[2] = (pixel >> 16) & 0xff;
                       p += mTarget->pitch;
                   }
-              } 
-          } break;
-  
+              }
+              break;
+
           case 4:
-          {
               for (;y1 <= y2; ++y1)
               {
                   if (mAlpha)
@@ -421,9 +480,10 @@ namespace gcn
                   }
                   p += mTarget->pitch;
               }
-          } break;
+              break;
+              
         } // end switch
-    
+        
         SDL_UnlockSurface(mTarget);
     }
 
@@ -438,7 +498,7 @@ namespace gcn
         drawHLine(x1, y2, x2);
 
         drawVLine(x1, y1, y2);
-        drawVLine(x2, y1, y2);    
+        drawVLine(x2, y1, y2);
     }
 
     void SDLGraphics::drawLine(int x1, int y1, int x2, int y2)
@@ -454,14 +514,20 @@ namespace gcn
             return;
         }
 
-        ClipRectangle top = mClipStack.top();
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
         x1 += top.xOffset;
         y1 += top.yOffset;
         x2 += top.xOffset;
         y2 += top.yOffset;
 
         // Draw a line with Bresenham
-        
+
         int dx = ABS(x2 - x1);
         int dy = ABS(y2 - y1);
 
@@ -477,16 +543,16 @@ namespace gcn
                 // swap y1, y2
                 y1 ^= y2;
                 y2 ^= y1;
-                y1 ^= y2;   
+                y1 ^= y2;
             }
 
             if (y1 < y2)
             {
                 int y = y1;
                 int p = 0;
-            
+
                 for (int x = x1; x <= x2; x++)
-                {            
+                {
                     if (top.isPointInRect(x, y))
                     {
                         if (mAlpha)
@@ -494,13 +560,13 @@ namespace gcn
                             SDLputPixelAlpha(mTarget, x, y, mColor);
                         }
                         else
-                        {            
+                        {
                             SDLputPixel(mTarget, x, y, mColor);
                         }
                     }
-                
+
                     p += dy;
-                
+
                     if (p * 2 >= dx)
                     {
                         y++;
@@ -512,9 +578,9 @@ namespace gcn
             {
                 int y = y1;
                 int p = 0;
-            
+
                 for (int x = x1; x <= x2; x++)
-                {            
+                {
                     if (top.isPointInRect(x, y))
                     {
                         if (mAlpha)
@@ -522,19 +588,19 @@ namespace gcn
                             SDLputPixelAlpha(mTarget, x, y, mColor);
                         }
                         else
-                        {            
+                        {
                             SDLputPixel(mTarget, x, y, mColor);
                         }
                     }
-                
+
                     p += dy;
-                
+
                     if (p * 2 >= dx)
                     {
                         y--;
                         p -= dx;
                     }
-                }            
+                }
             }
         }
         else
@@ -549,16 +615,16 @@ namespace gcn
                 // swap x1, x2
                 x1 ^= x2;
                 x2 ^= x1;
-                x1 ^= x2;   
+                x1 ^= x2;
             }
 
             if (x1 < x2)
             {
                 int x = x1;
                 int p = 0;
-            
+
                 for (int y = y1; y <= y2; y++)
-                {            
+                {
                     if (top.isPointInRect(x, y))
                     {
                         if (mAlpha)
@@ -566,13 +632,13 @@ namespace gcn
                             SDLputPixelAlpha(mTarget, x, y, mColor);
                         }
                         else
-                        {            
+                        {
                             SDLputPixel(mTarget, x, y, mColor);
                         }
                     }
-                
+
                     p += dx;
-                
+
                     if (p * 2 >= dy)
                     {
                         x++;
@@ -584,9 +650,9 @@ namespace gcn
             {
                 int x = x1;
                 int p = 0;
-            
+
                 for (int y = y1; y <= y2; y++)
-                {            
+                {
                     if (top.isPointInRect(x, y))
                     {
                         if (mAlpha)
@@ -594,42 +660,49 @@ namespace gcn
                             SDLputPixelAlpha(mTarget, x, y, mColor);
                         }
                         else
-                        {            
+                        {
                             SDLputPixel(mTarget, x, y, mColor);
                         }
                     }
-                
+
                     p += dx;
-                
+
                     if (p * 2 >= dy)
                     {
                         x--;
                         p -= dy;
                     }
-                }            
+                }
             }
-        }    
+        }
     }
-    
+
     void SDLGraphics::setColor(const Color& color)
     {
-        mColor = color;    
+        mColor = color;
 
-        mAlpha = color.a != 255;        
+        mAlpha = color.a != 255;
     }
 
-    const Color& SDLGraphics::getColor()
+    const Color& SDLGraphics::getColor() const
     {
-        return mColor;    
+        return mColor;
     }
 
-    void SDLGraphics::drawSDLSurface(SDL_Surface* surface, SDL_Rect source,
+    void SDLGraphics::drawSDLSurface(SDL_Surface* surface,
+                                     SDL_Rect source,
                                      SDL_Rect destination)
     {
-        ClipRectangle top = mClipStack.top();
+        if (mClipStack.empty())
+        {
+            throw GCN_EXCEPTION("Clip stack is empty, perhaps you called a draw funtion outside of _beginDraw() and _endDraw()?");
+        }
+
+        const ClipRectangle& top = mClipStack.top();
+
         destination.x += top.xOffset;
         destination.y += top.yOffset;
-    
-        SDL_BlitSurface(surface, &source, mTarget, &destination);        
+
+        SDL_BlitSurface(surface, &source, mTarget, &destination);
     }
 }
