@@ -1,12 +1,14 @@
-/*      _______   __   __   __   ______   __   __   _______   __   __                 
- *     / _____/\ / /\ / /\ / /\ / ____/\ / /\ / /\ / ___  /\ /  |\/ /\                
- *    / /\____\// / // / // / // /\___\// /_// / // /\_/ / // , |/ / /                 
- *   / / /__   / / // / // / // / /    / ___  / // ___  / // /| ' / /                  
- *  / /_// /\ / /_// / // / // /_/_   / / // / // /\_/ / // / |  / /                   
- * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /                    
- * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/                      
+/*      _______   __   __   __   ______   __   __   _______   __   __
+ *     / _____/\ / /\ / /\ / /\ / ____/\ / /\ / /\ / ___  /\ /  |\/ /\
+ *    / /\____\// / // / // / // /\___\// /_// / // /\_/ / // , |/ / /
+ *   / / /__   / / // / // / // / /    / ___  / // ___  / // /| ' / /
+ *  / /_// /\ / /_// / // / // /_/_   / / // / // /\_/ / // / |  / /
+ * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /
+ * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/
  *
- * Copyright (c) 2004, 2005 darkbits                        Js_./
+ * Copyright (c) 2004, 2005, 2006, 2007 Olof Naessn and Per Larsson
+ *
+ *                                                         Js_./
  * Per Larsson a.k.a finalman                          _RqZ{a<^_aa
  * Olof Naessn a.k.a jansem/yakslem                _asww7!uY`>  )\a//
  *                                                 _Qhm`] _f "'c  1!5m
@@ -52,70 +54,19 @@
  * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "guichan/exception.hpp"
 #include "guichan/widgets/dropdown.hpp"
 
+#include "guichan/exception.hpp"
+#include "guichan/font.hpp"
+#include "guichan/graphics.hpp"
+#include "guichan/key.hpp"
+#include "guichan/listmodel.hpp"
+#include "guichan/mouseinput.hpp"
+#include "guichan/widgets/listbox.hpp"
+#include "guichan/widgets/scrollarea.hpp"
+
 namespace gcn
-{  
-    DropDown::DropDown()
-    {
-        mDroppedDown = false;
-        mPushed = false;
-    
-        setWidth(100);
-        setFocusable(true);
-
-        mDefaultScrollArea = new ScrollArea();
-        mDefaultScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
-        mDefaultListBox = new ListBox();
-
-        mScrollArea = mDefaultScrollArea;
-        mScrollArea->_setFocusHandler(&mFocusHandler);
-        mScrollArea->_setParent(this);
-        
-        mListBox = mDefaultListBox;    
-        mListBox->addActionListener(this);
-        mScrollArea->setContent(mListBox);
-        
-        addMouseListener(this);
-        addKeyListener(this);
-        adjustHeight();
-        setBorderSize(1);        
-    }
-    
-    DropDown::DropDown(ListModel *listModel)
-    {
-        setWidth(100);
-        setFocusable(true);
-        mDroppedDown = false;
-        mPushed = false;    
-    
-        mDefaultScrollArea = new ScrollArea();
-        mDefaultScrollArea->setHorizontalScrollPolicy(ScrollArea::SHOW_NEVER);
-        mDefaultListBox = new ListBox();
-
-        mScrollArea = mDefaultScrollArea;
-        mScrollArea->_setParent(this);
-        mListBox = mDefaultListBox;
-        mListBox->addActionListener(this);
-            
-        mScrollArea->setContent(mListBox);
-        mScrollArea->_setFocusHandler(&mFocusHandler);
-        mScrollArea->_setParent(this);
-            
-        setListModel(listModel);
-    
-        if (mListBox->getSelected() < 0)
-        {
-            mListBox->setSelected(0);
-        }
-
-        addMouseListener(this);
-        addKeyListener(this);
-        adjustHeight();
-        setBorderSize(1);        
-    }
-  
+{
     DropDown::DropDown(ListModel *listModel,
                        ScrollArea *scrollArea,
                        ListBox *listBox)
@@ -124,18 +75,36 @@ namespace gcn
         setFocusable(true);
         mDroppedDown = false;
         mPushed = false;
-    
-        mDefaultScrollArea = NULL;
-        mDefaultListBox = NULL;
+        mIsDragged = false;
 
-        mScrollArea = scrollArea;
-        mScrollArea->_setFocusHandler(&mFocusHandler);
+        setInternalFocusHandler(&mInternalFocusHandler);
+
+        mInternalScrollArea = (scrollArea == NULL);
+        mInternalListBox = (listBox == NULL);
+
+        if (mInternalScrollArea)
+        {
+            mScrollArea = new ScrollArea();
+        }
+        else
+        {
+            mScrollArea = scrollArea;
+        }
+
+        if (mInternalListBox)
+        {
+            mListBox = new ListBox();
+        }
+        else
+        {
+            mListBox = listBox;
+        }
 
-        mListBox = listBox;    
-        mListBox->addActionListener(this);
         mScrollArea->setContent(mListBox);
-        mScrollArea->_setParent(this);
-            
+        add(mScrollArea);
+
+        mListBox->addActionListener(this);
+
         setListModel(listModel);
 
         if (mListBox->getSelected() < 0)
@@ -145,51 +114,32 @@ namespace gcn
 
         addMouseListener(this);
         addKeyListener(this);
+        addFocusListener(this);
+
         adjustHeight();
-        setBorderSize(1);        
+        setBorderSize(1);
     }
 
     DropDown::~DropDown()
-    {      
-        if (mScrollArea != NULL)
-        {
-            mScrollArea->_setFocusHandler(NULL);
-        }
-
-        if (mDefaultScrollArea != NULL)
-        {
-            delete mDefaultScrollArea;
-        }
-
-        if (mDefaultListBox != NULL)
-        {
-            delete mDefaultListBox;
-        }
-
+    {
         if (widgetExists(mListBox))
         {
             mListBox->removeActionListener(this);
         }
-    }
-  
-    void DropDown::logic()
-    {
-        if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
+
+        if (mInternalScrollArea)
         {
-            throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
-        }
-      
-        mScrollArea->logic();
-        mFocusHandler.applyChanges();
-    }
-    
-    void DropDown::draw(Graphics* graphics)
-    {     
-        if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
-        {
-            throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
+            delete mScrollArea;
         }
 
+        if (mInternalListBox)
+        {
+            delete mListBox;
+        }
+    }
+
+    void DropDown::draw(Graphics* graphics)
+    {
         int h;
 
         if (mDroppedDown)
@@ -200,39 +150,39 @@ namespace gcn
         {
             h = getHeight();
         }
-    
+
         int alpha = getBaseColor().a;
         Color faceColor = getBaseColor();
         faceColor.a = alpha;
         Color highlightColor = faceColor + 0x303030;
         highlightColor.a = alpha;
-        Color shadowColor = faceColor - 0x303030;      
-        shadowColor.a = alpha;    
-        
+        Color shadowColor = faceColor - 0x303030;
+        shadowColor.a = alpha;
+
 
         graphics->setColor(getBackgroundColor());
         graphics->fillRectangle(Rectangle(0, 0, getWidth(), h));
-   
+
         graphics->setColor(getForegroundColor());
         graphics->setFont(getFont());
-    
+
+        if (isFocused())
+        {
+            graphics->setColor(getSelectionColor());
+            graphics->fillRectangle(Rectangle(0, 0, getWidth() - h, h));
+            graphics->setColor(getForegroundColor());
+        }
+
         if (mListBox->getListModel() && mListBox->getSelected() >= 0)
         {
             graphics->drawText(mListBox->getListModel()->getElementAt(mListBox->getSelected()), 1, 0);
         }
 
-        if (hasFocus())
-        {
-            graphics->drawRectangle(Rectangle(0, 0, getWidth() - h, h));
-        }
-        
         drawButton(graphics);
-            
+
          if (mDroppedDown)
          {
-             graphics->pushClipArea(mScrollArea->getDimension());
-             mScrollArea->draw(graphics);
-             graphics->popClipArea();
+             drawChildren(graphics);
 
             // Draw two lines separating the ListBox with se selected
             // element view.
@@ -240,7 +190,7 @@ namespace gcn
             graphics->drawLine(0, h, getWidth(), h);
             graphics->setColor(shadowColor);
             graphics->drawLine(0, h + 1,getWidth(),h + 1);
-         }        
+         }
     }
 
     void DropDown::drawBorder(Graphics* graphics)
@@ -254,7 +204,7 @@ namespace gcn
         highlightColor.a = alpha;
         shadowColor = faceColor - 0x303030;
         shadowColor.a = alpha;
-        
+
         unsigned int i;
         for (i = 0; i < getBorderSize(); ++i)
         {
@@ -262,8 +212,8 @@ namespace gcn
             graphics->drawLine(i,i, width - i, i);
             graphics->drawLine(i,i + 1, i, height - i - 1);
             graphics->setColor(highlightColor);
-            graphics->drawLine(width - i,i + 1, width - i, height - i); 
-            graphics->drawLine(i,height - i, width - i - 1, height - i); 
+            graphics->drawLine(width - i,i + 1, width - i, height - i);
+            graphics->drawLine(i,height - i, width - i - 1, height - i);
         }
     }
 
@@ -272,7 +222,7 @@ namespace gcn
         Color faceColor, highlightColor, shadowColor;
         int offset;
         int alpha = getBaseColor().a;
-        
+
         if (mPushed)
         {
             faceColor = getBaseColor() - 0x303030;
@@ -304,7 +254,7 @@ namespace gcn
             h = getHeight();
         }
         int x = getWidth() - h;
-        int y = 0;    
+        int y = 0;
 
         graphics->setColor(faceColor);
         graphics->fillRectangle(Rectangle(x+1, y+1, h-2, h-2));
@@ -318,7 +268,7 @@ namespace gcn
         graphics->drawLine(x+1, y+h-1, x+h-2, y+h-1);
 
         graphics->setColor(getForegroundColor());
-    
+
         int i;
         int hh = h / 3;
         int hx = x + h / 2;
@@ -329,147 +279,147 @@ namespace gcn
                                hy - i + offset,
                                hx + i + offset,
                                hy - i + offset);
-        }        
+        }
     }
-    
+
     int DropDown::getSelected()
     {
-        if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
-        {
-            throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
-        }
-
-        return mListBox->getSelected();        
+        return mListBox->getSelected();
     }
-    
+
     void DropDown::setSelected(int selected)
     {
-        if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
-        {
-            throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
-        }
-
         if (selected >= 0)
         {
             mListBox->setSelected(selected);
-        }        
-    }
-    
-    void DropDown::keyPress(const Key& key)
-    {
-        if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
-        {
-            throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
         }
+    }
+
+    void DropDown::keyPressed(KeyEvent& keyEvent)
+    {
+        Key key = keyEvent.getKey();
 
         if ((key.getValue() == Key::ENTER || key.getValue() == Key::SPACE)
             && !mDroppedDown)
         {
             dropDown();
-        }        
+            keyEvent.consume();
+        }
+        else if (key.getValue() == Key::UP)
+        {
+            setSelected(getSelected() - 1);
+            keyEvent.consume();
+        }
+        else if (key.getValue() == Key::DOWN)
+        {
+            setSelected(getSelected() + 1);
+            keyEvent.consume();
+        }
     }
-  
-    void DropDown::mousePress(int x, int y, int button)
-    {
-        if (button == MouseInput::LEFT && hasMouse() && !mDroppedDown)
+
+    void DropDown::mousePressed(MouseEvent& mouseEvent)
+    {        
+        // If we have a mouse press on the widget.
+        if (0 <= mouseEvent.getY()
+            && mouseEvent.getY() < getHeight()
+            && mouseEvent.getX() >= 0
+            && mouseEvent.getX() < getWidth()
+            && mouseEvent.getButton() == MouseEvent::LEFT
+            && !mDroppedDown
+            && mouseEvent.getSource() == this)
         {
             mPushed = true;
             dropDown();
+            requestModalMouseInputFocus();
         }
         // Fold up the listbox if the upper part is clicked after fold down
-        else if (button == MouseInput::LEFT && hasMouse() && mDroppedDown
-                 && y < mOldH)
+        else if (0 <= mouseEvent.getY()
+                 && mouseEvent.getY() < mOldH
+                 && mouseEvent.getX() >= 0
+                 && mouseEvent.getX() < getWidth()
+                 && mouseEvent.getButton() == MouseEvent::LEFT
+                 && mDroppedDown
+                 && mouseEvent.getSource() == this)
         {
+            mPushed = false;
+            foldUp();
+            releaseModalMouseInputFocus();
+        }
+        // If we have a mouse press outside the widget
+        else if (0 > mouseEvent.getY()
+                 || mouseEvent.getY() >= getHeight()
+                 || mouseEvent.getX() < 0
+                 || mouseEvent.getX() >= getWidth())
+        {
+            mPushed = false;
             foldUp();
         }
-        else if (!hasMouse())
-        {
-            foldUp();
-        }        
     }
 
-    void DropDown::mouseRelease(int x, int y, int button)
+    void DropDown::mouseReleased(MouseEvent& mouseEvent)
     {
-        if (button == MouseInput::LEFT)
-        {      
+        if (mIsDragged)
+        {
             mPushed = false;
         }
+
+        // Released outside of widget. Can happen when we have modal input focus.
+        if (0 > mouseEvent.getY()
+            || mouseEvent.getY() >= getHeight()
+            || mouseEvent.getX() < 0
+            || mouseEvent.getX() >= getWidth()
+            && mouseEvent.getButton() == MouseEvent::LEFT
+            && hasModalMouseInputFocus())
+        {
+            releaseModalMouseInputFocus();
+
+            if (mIsDragged)
+            {
+                foldUp();
+            }
+        }
+        else if (mouseEvent.getButton() == MouseEvent::LEFT)
+        {
+            mPushed = false;
+        }
+
+        mIsDragged = false;
     }
-  
+
+    void DropDown::mouseDragged(MouseEvent& mouseEvent)
+    {
+        mIsDragged = true;
+
+        mouseEvent.consume();
+    }
+
     void DropDown::setListModel(ListModel *listModel)
     {
-        if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
-        {
-            throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
-        }
-
         mListBox->setListModel(listModel);
-    
+
         if (mListBox->getSelected() < 0)
         {
             mListBox->setSelected(0);
         }
-    
-        adjustHeight();        
+
+        adjustHeight();
     }
-  
+
     ListModel *DropDown::getListModel()
     {
-        if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
-        {
-            throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
-        }
-      
-        return mListBox->getListModel();        
+        return mListBox->getListModel();
     }
 
-    void DropDown::setScrollArea(ScrollArea *scrollArea)
-    {
-        mScrollArea->_setFocusHandler(NULL);
-        mScrollArea->_setParent(NULL);
-        mScrollArea = scrollArea;
-        mScrollArea->_setFocusHandler(&mFocusHandler);
-        mScrollArea->setContent(mListBox);
-        mScrollArea->_setParent(this);
-        adjustHeight();        
-    }
-
-    ScrollArea *DropDown::getScrollArea()
-    {
-        return mScrollArea;        
-    }
-  
-    void DropDown::setListBox(ListBox *listBox)
-    {
-        listBox->setSelected(mListBox->getSelected());
-        listBox->setListModel(mListBox->getListModel());
-        listBox->addActionListener(this);
-
-        if (mScrollArea->getContent() != NULL)
-        {
-            mListBox->removeActionListener(this);
-        }
-        
-        mListBox = listBox;
-    
-        mScrollArea->setContent(mListBox);
-    
-        if (mListBox->getSelected() < 0)
-        {
-            mListBox->setSelected(0);
-        }       
-    }
-
-    ListBox *DropDown::getListBox()
-    {
-        return mListBox;        
-    }
-  
     void DropDown::adjustHeight()
     {
-        if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
+        if (mScrollArea == NULL)
         {
-            throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
+            throw GCN_EXCEPTION("Scroll area has been deleted.");
+        }
+
+        if (mListBox == NULL)
+        {
+            throw GCN_EXCEPTION("List box has been deleted.");
         }
 
         int listBoxHeight = mListBox->getHeight();
@@ -479,11 +429,11 @@ namespace gcn
 
         // The addition/subtraction of 2 compensates for the seperation lines
         // seperating the selected element view and the scroll area.
-        
+
         if (mDroppedDown && getParent())
         {
-            int h = getParent()->getHeight() - getY();
-      
+            int h = getParent()->getChildrenArea().height - getY();
+
             if (listBoxHeight > h - h2 - 2)
             {
                 mScrollArea->setHeight(h - h2 - 2);
@@ -497,11 +447,13 @@ namespace gcn
         }
 
         mScrollArea->setWidth(getWidth());
-        mScrollArea->setPosition(0, h2 + 2);        
+        // Resize the ListBox to exactly fit the ScrollArea.
+        mListBox->setWidth(mScrollArea->getChildrenArea().width);
+        mScrollArea->setPosition(0, 0);
     }
 
     void DropDown::dropDown()
-    {        
+    {
         if (!mDroppedDown)
         {
             mDroppedDown = true;
@@ -513,168 +465,135 @@ namespace gcn
                 getParent()->moveToTop(this);
             }
         }
-        
-        mFocusHandler.requestFocus(mScrollArea->getContent());
+
+        mListBox->requestFocus();
     }
-    
+
     void DropDown::foldUp()
-    {        
+    {
         if (mDroppedDown)
         {
             mDroppedDown = false;
-            mFocusHandler.focusNone();
             adjustHeight();
-        }        
-    }
-
-    void DropDown::_keyInputMessage(const KeyInput& keyInput)
-    {        
-        if (mDroppedDown)
-        {
-            if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
-            {
-                throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
-            }
-
-            if (mFocusHandler.getFocused() != NULL)
-            {
-                mFocusHandler.getFocused()->_keyInputMessage(keyInput);
-            }
+            mInternalFocusHandler.focusNone();
         }
-        else
-        {
-            BasicContainer::_keyInputMessage(keyInput);
-        }        
-    }
-    
-    void DropDown::_mouseInputMessage(const MouseInput &mouseInput)
-    {            
-        BasicContainer::_mouseInputMessage(mouseInput);
-
-        if (mDroppedDown)
-        {
-            if (mScrollArea == NULL || mScrollArea->getContent() == NULL)
-            {
-                throw GCN_EXCEPTION("ScrollArea or ListBox is NULL.");
-            }
-                    
-            if (mouseInput.y >= mOldH)
-            {
-                MouseInput mi = mouseInput;
-                mi.y -= mScrollArea->getY();        
-                mScrollArea->_mouseInputMessage(mi);
-
-                if (mListBox->hasFocus())
-                {
-                    mi.y -= mListBox->getY();
-                    mListBox->_mouseInputMessage(mi);
-                }
-            }
-        }        
     }
 
-    void DropDown::lostFocus()
-    {
-        foldUp();        
-    }
-
-    void DropDown::moveToTop(Widget* widget)
-    {
-        if (getParent())
-        {
-            getParent()->moveToTop(this);
-        }        
-    }
-
-    void DropDown::moveToBottom(Widget* widget)
-    {
-        if (getParent())
-        {
-            getParent()->moveToBottom(this);
-        }        
-    }
-
-    void DropDown::_announceDeath(Widget* widget)
-    {        
-        if (widget == mScrollArea)
-        {            
-            mScrollArea = NULL;
-        }
-        else
-        {
-            throw GCN_EXCEPTION("Death announced for unknown widget..");
-        }        
-    }
-
-    void DropDown::action(const std::string& eventId)
+    void DropDown::focusLost(const Event& event)
     {
         foldUp();
-        generateAction();        
-    } 
+        mInternalFocusHandler.focusNone();
+    }
 
-    void DropDown::getDrawSize(int& width, int& height, Widget* widget)
-    {
-        if (widget == mScrollArea)
+
+    void DropDown::death(const Event& event)
+    {        
+        if (event.getSource() == mScrollArea)
         {
-            if (mDroppedDown)
-            {
-                height = getHeight() - mOldH;
-                width = getWidth();
-            }
-            else
-            {
-                width = height = 0;
-            }
+            mScrollArea = NULL;
         }
-        else
+
+        BasicContainer::death(event);
+    }
+
+    void DropDown::action(const ActionEvent& actionEvent)
+    {
+        foldUp();
+        releaseModalMouseInputFocus();
+        generateAction();
+    }
+
+    Rectangle DropDown::getChildrenArea()
+    {
+        if (mDroppedDown)
         {
-            throw GCN_EXCEPTION("DropDown::getDrawSize. widget is not the ScrollArea (wieeerd...)");
-        }        
+            return Rectangle(0, mOldH + 2, getWidth(), getHeight() - mOldH);
+        }
+
+        return Rectangle();
     }
 
     void DropDown::setBaseColor(const Color& color)
     {
-        if (mDefaultScrollArea == mScrollArea && mScrollArea != NULL)
+        if (mInternalScrollArea)
         {
             mScrollArea->setBaseColor(color);
         }
 
-        if (mDefaultListBox == mListBox && mListBox != NULL)
+        if (mInternalListBox)
         {
             mListBox->setBaseColor(color);
         }
-                
+
         Widget::setBaseColor(color);
     }
 
     void DropDown::setBackgroundColor(const Color& color)
     {
-        if (mDefaultScrollArea == mScrollArea && mScrollArea != NULL)
+        if (mInternalScrollArea)
         {
             mScrollArea->setBackgroundColor(color);
         }
 
-        if (mDefaultListBox == mListBox && mListBox != NULL)
+        if (mInternalListBox)
         {
             mListBox->setBackgroundColor(color);
         }
-                
+
         Widget::setBackgroundColor(color);
     }
 
     void DropDown::setForegroundColor(const Color& color)
     {
-        if (mDefaultScrollArea == mScrollArea && mScrollArea != NULL)
+        if (mInternalScrollArea)
         {
             mScrollArea->setForegroundColor(color);
         }
 
-        if (mDefaultListBox == mListBox && mListBox != NULL)
+        if (mInternalListBox)
         {
             mListBox->setForegroundColor(color);
         }
-                
+
         Widget::setForegroundColor(color);
-    }     
+    }
+
+	void DropDown::setFont(Font *font)
+	{
+		if (mInternalScrollArea)
+        {
+            mScrollArea->setFont(font);
+        }
+
+        if (mInternalListBox)
+        {
+            mListBox->setFont(font);
+        }
+
+        Widget::setFont(font);
+	}
+
+	void DropDown::mouseWheelMovedUp(MouseEvent& mouseEvent)
+	{
+        if (isFocused() && mouseEvent.getSource() == this)
+        {                   
+            mouseEvent.consume();
+
+            if (mListBox->getSelected() > 0)
+            {
+                mListBox->setSelected(mListBox->getSelected() - 1);
+            }
+        }
+    }
+
+    void DropDown::mouseWheelMovedDown(MouseEvent& mouseEvent)
+    {
+        if (isFocused() && mouseEvent.getSource() == this)
+        {            
+            mouseEvent.consume();
+
+            mListBox->setSelected(mListBox->getSelected() + 1);
+        }
+    }
 }
 
